/*
  This file is a portion of Hsim 0.2
 
  Hsim is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 */

/*
 * Vent model
 *
 * This routine computes N2O flow rate through the tank vent.
 *
 * The flow of N2O through the vent is modeled as an ideal gas
 * flowing through the vent.
 *
 * The c* equation is taken from Sutton, 7'th edition, 3-32.
 *
 * DYNAMIC INPUTS:
 *	tank_temperature
 *	tank_pressure
 *
 * STATIC INPUTS:
 *	vent_area
 *	vent_cd
 *
 * OUTPUTS:
 * 	n2o_vent_rate
 */

/*
 * IMPLEMENTATION NOTE:
 *
 * This model assumes constant Cp/Cv as the ideal gas flows through
 * the nozzle.  The assumption is not valid for N2O.  However, we
 * believe that the inaccuracies show up primarily in the amount of
 * thrust generated by the vent, not in the mass flow through the vent.
 *
 * Until we have time to research cold gas thrusters in more depth
 * we will stay with this model.
 */


#include <math.h>
#include <stdio.h>
#include "state.h"
#include "linkage.h"

extern char *myname;

void
vent()
{
	double k, R, T1;
	double cstar;

	T1 = tank_temperature;
	R = ideal_gas_constant;
	k = cpcv(tank_temperature);
	
	cstar = sqrt(k * R * T1) / (
		k * sqrt(pow(2 / (k + 1), (k + 1) / k - 1)) );

	n2o_vent_rate = tank_pressure * vent_area * vent_cd / cstar;
}
